#!/usr/bin/env ruby

module LifePex
end

require "sinatra/base" # web
require "sinatra/cookies"
require "slim"
require "sinatra/namespace" # api
require "doc_my_routes" # api doc
require "bcrypt" # security
require "jwt"
require "rack/csrf"
require "securerandom"
require "sequel" # db
require "json" # helpers
require "active_support"
require "active_support/core_ext"
require "pry" # debug

require_relative "./utils/url.rb"
require_relative "./utils/boot_framework"

LifePex::BootFramework.boot_application do
  harshly_need_env "LIFEPEX_DB"
  kindly_ask_env "LIFEPEX_BASE_URL"
  kindly_ask_env "LIFEPEX_SECRET" do
    warning "\"LIFEPEX_SECRET\" will be randomly generated as a fallback"
    ENV["LIFEPEX_SECRET"] = SecureRandom.hex(64)
  end
  harshly_do env["LIFEPEX_SECRET"].size < 8 do
    error "Your secret is NOT very secret, fix this (at least 8 hexadigits entropy)"
  end
end.finish!

module LifePex
  DB = Sequel.connect ENV["LIFEPEX_DB"]
  BASE_URL = ENV["LIFEPEX_BASE_URL"] || "http://localhost:4567"
  SECRET = ENV["LIFEPEX_SECRET"]
  CODE_VERSION = DB[:meta].first[:code_version]
  CODE_DATE = DB[:meta].first[:code_date]
  include LifePex::Utils::Url

  module Systems
  end
end

require_relative "./utils/json_api.rb"
require_relative "./utils/string.rb"

require_relative "./achievements/dsl.rb"
require_relative "./achievements/achievement.rb"

require_relative "./models/level.rb"
require_relative "./models/user.rb"
require_relative "./models/pex.rb"
require_relative "./models/user_pex.rb"
require_relative "./models/achievement.rb"

require_relative "./systems/csrf.rb"
require_relative "./systems/api_list.rb"
require_relative "./systems/auth.rb"
require_relative "./systems/user.rb"
require_relative "./systems/pex.rb"
require_relative "./systems/pex2.rb"
require_relative "./systems/achievement.rb"

class LifePex::Systems::PublicSystem < Sinatra::Base
  set :public_folder, "public"
end

class LifePex::Systems::BaseSystem < Sinatra::Base
  DocMyRoutes.configure do |config|
    config.title = "LifePex"
    config.description = "LifePex JSON REST API documentation"
  end

  set :session_secret, LifePex::SECRET
  enable :sessions

  use Rack::Csrf, skip: ["*:/api*"] if ENV["LIFEPEX_ENV"] != "test"

  use LifePex::Systems::AuthSystem
  use LifePex::Systems::PublicSystem
  use LifePex::Systems::UserSystem
  use LifePex::Systems::PexSystem
  use LifePex::Systems::Pex2System
  use LifePex::Systems::Achievement

  include JSON::API
  not_found do
    if accept_json?
      { message: "No route or entity \"#{request.path}\" found" }.to_json
    else
      raise
    end
  end

  get "/api/meta/v1", provides: "json" do
    LifePex::Systems::ApiList.get_all_api_routes.to_json
  end

  set :bind, ENV["LIFEPEX_BIND"] || "127.0.0.1"

  APP_ENV = ENV.fetch("LIFEPEX_ENV") { "development" }
  set :environment, APP_ENV
  ENV["RACK_ENV"] = APP_ENV

  run! if app_file == $0
end
